using System;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;
using SimpleJSON;
using UnityEngine.UI;
using static MainUI;

public class ActionListManager : Persistence
{
    private MainUI mainUI;
    private int newActionCounter = 1;
    public bool lockedOnTransitionAction = false;
    public string lockTransitionAnimationName;
    private float timeSinceLastUnlockCheck = 0f;
    public string lastVoxtaState;

    public UIAssistProvider uIAssistProvider;

    private ActionItem deferredActionItem;
    public ActionItem lastAction { get; private set; }
    private List<Trigger> activeTriggers = new List<Trigger>();
    private bool scheduleRefreshUI;
    private UnityEventsListener PluginUIEventsListener { get; set; }
    private JSONStorableString triggerSelectedAction = new JSONStorableString("TriggerSelectedActionChange", "");

    public void OnUIAButtonListDataRequested(Dictionary<string, object> buttonListData)
    {
        uIAssistProvider.OnUIAButtonListDataRequested(buttonListData);
    }

    public void CleanChat()
    {
        UI.inspectorUI.messages.Clear();
        UI.inspectorUI.RefreshUI();
    }

    public override void Init()
    {
        uIAssistProvider = new UIAssistProvider(OnTriggerSelectedActionChange);
        SuperController.singleton.BroadcastMessage("OnActionsProviderAvailable", this, SendMessageOptions.DontRequireReceiver);

        try
        {
            InitStorables();
            InitPluginActions();
        }
        catch (Exception e)
        {
            SuperController.LogError($"{nameof(ActionListManager)}.{nameof(Init)}: {e}");
        }
    }
    public override void InitUI()
    {
        base.InitUI();
        if (!UITransform || PluginUIEventsListener)
        {
            return;
        }

        PluginUIEventsListener = UITransform.gameObject.AddComponent<UnityEventsListener>();
        PluginUIEventsListener.EnabledHandlers += () =>
        {
            if (scheduleRefreshUI)
            {
                scheduleRefreshUI = false;
                RefreshUI();
            }
        };

        InitOptionsUI();

        // TODO do it better
        mainUI = UI.mainUI;

        RefreshUI();

        //set background color
        UITransform.GetComponentInChildren<MVRScriptUI>().rightUIContent.parent.parent.parent.transform.GetComponent<Image>().color = Colors.MAIN_WINDOW;
        UITransform.GetComponentInChildren<MVRScriptUI>().rightUIContent.parent.parent.parent.parent.transform.GetComponent<Image>().color = Colors.MAIN_WINDOW_BACK;
    }

    public void RefreshActionUI()
    {
        if (!ShouldDeferRefresh() && actionsDictionary.Count() > 0)
        {
            UI.actionTextFieldsUI.RefreshUI();
            UI.linkedActionsUI.RefreshUI();
            UI.newLinkedActionsUI.RefreshUI();
            UI.inspectorUI.RefreshUI();
            UI.bulkActionManagerUI.RefreshUI();
            UI.groupUI.RefreshUI();
            UI.layerUI.RefreshUI();
        }
    }

    private void InitOptionsUI()
    {
        Utils.OnInitUI(CreateUIElement);
        UI.Init(this);
    }

    private void OnExecuteActionButtonClicked()
    {
        OnTriggerSelectedActionChange(GetChosenAction());
        OnTriggerStateChange();
    }

    private void OnDuplicateActionButtonClicked()
    {
        string actionToDuplicate = GetChosenAction();
        if (string.IsNullOrEmpty(actionToDuplicate))
        {
            return;
        }

        if (actionsDictionary.ContainsKey(GetChosenAction()))
        {
            ActionItem duplicatedAction = new ActionItem(GetUniqueDuplicateName(GetChosenAction()), actionsDictionary[actionToDuplicate]);
            actionsDictionary.Add(duplicatedAction.Name, duplicatedAction);

            SetChoosenAction(duplicatedAction.Name);
            UI.linkedActionsUI.RefreshUI();
        }
        else
        {
            SuperController.LogError($"Action '{actionToDuplicate}' not found in dictionary.");
        }
    }

    private string GetUniqueDuplicateName(string baseName)
    {
        int counter = 1;
        string newName;
        do
        {
            newName = $"{baseName} Copy_{counter++}";
        } while (actionsDictionary.ContainsKey(newName));
        return newName;
    }

    private void OnRemoveActionButtonClicked()
    {
        string actionToRemove = GetChosenAction();
        RemoveAction(actionToRemove);
    }

    public void RemoveAction(string actionToRemove)
    {
        if (string.IsNullOrEmpty(actionToRemove))
        {
            return;
        }

        if (actionsDictionary.ContainsKey(actionToRemove))
        {
            foreach (var key in new List<string>(actionsDictionary.Keys))
            {
                actionsDictionary[key].LinkedActions.Remove(actionToRemove);
            }

            int indexToRemove = actionsChooser.choices.IndexOf(actionToRemove);
            int nextIndex = 0;
            if (indexToRemove >= 0 && indexToRemove < actionsChooser.choices.Count - 1)
            {
                nextIndex = indexToRemove;
            }

            actionsDictionary.Remove(actionToRemove);
            if (lastAction != null && lastAction.Name == actionToRemove)
            {
                lastAction = null;
            }
            if (lastRoutingAction != null && lastRoutingAction.Name == actionToRemove)
            {
                lastRoutingAction = null;
            }

            actionsChooser.choices = actionsDictionary.Keys.ToList();

            if (actionsChooser.choices.Count > 0)
            {
                actionsChooser.val = actionsChooser.choices[nextIndex];
            }
            else
            {
                UI.actionTextFieldsUI.resetAllFields();
                UI.linkedActionsUI.resetAllFields();
                SetChoosenAction("");
                UI.actionTextFieldsUI.RemoveAllUI();
                UI.linkedActionsUI.RemoveAllUI();
                return;
            }
            UI.inspectorUI.RefreshUI();
            uIAssistProvider.RemoveAction(actionToRemove);
        }
        else
        {
            SuperController.LogError("Action to remove not found in dictionary.");
        }
    }

    private void OnAddActionButtonClicked()
    {
        string nameValue = "new action " + newActionCounter++;
        string actionValue = ActionNameUtils.FunctionFormatActionName(nameValue);
        string whenValue = "<type_in_trigger_condition>";

        ActionItem newActionItem = new ActionItem(nameValue, actionValue, whenValue, this);

        actionsDictionary.Add(nameValue, newActionItem);
        actionsChooser.choices = actionsDictionary.Keys.ToList();
        SetChoosenAction(nameValue);

        RefreshActionUI();
    }

    private void InitPluginActions()
    {
        RegisterAction(new JSONStorableAction("TriggerActionChange", OnTriggerActionChange));
        RegisterAction(new JSONStorableAction("TriggerStateChange", OnTriggerStateChange));
        RegisterAction(new JSONStorableAction("OnTriggerSpeak", OnTriggerSpeak));
        RegisterAction(new JSONStorableAction("TransitionActionFinished", OnTransitionActionFinished));
    }

    private void InitStorables()
    {
        triggerSelectedAction.setCallbackFunction = OnTriggerSelectedActionChange;
        RegisterString(triggerSelectedAction);
    }

    private void OnTransitionActionFinished()
    {
        if (lockedOnTransitionAction)
        {
            lockedOnTransitionAction = false;
            lockTransitionAnimationName = null;
            UpdateActionList(lastRoutingAction);
            UpdateCurrentContext(lastAction);
            RefreshUI();
            SetChoosenAction(lastRoutingAction.Name);
        }
    }
    private void OnTriggerStateChange()
    {
        string newVoxtaState = Voxta.GetState(this);

        if (lastVoxtaState == "listening" && newVoxtaState == "thinking")
        {
            UI.inspectorUI.OnIncomingUserMessage(Voxta.GetUserLastMessage(this));
        }

        lastVoxtaState = newVoxtaState;
        UI.inspectorUI.UpdateVoxtaState();
        if ((newVoxtaState == "idle" || newVoxtaState == "off") && deferredActionItem != null)
        {
            ExecuteAction(deferredActionItem);
            SetChooserToNewActionWhenLinkedToOthers(deferredActionItem);
            deferredActionItem = null;
        }
    }

    private void OnTriggerSpeak()
    {
        string characterMessage = Voxta.GetCharacterLastMessage(this);
        UI.inspectorUI.OnIncomingCharacterMessage(characterMessage);
    }

    private void OnTriggerSelectedActionChange(string name)
    {
        triggerSelectedAction.valNoCallback = "";
        if (!actionsDictionary.ContainsKey(name))
        {
            SuperController.LogError("There's no action with name: " + name);
            return;
        }
        if (actionsDictionary.Count > 0)
        {
            ExecuteAction(GetItemByName(name));
        }
    }

    private void OnTriggerActionChange()
    {
        string newAction = Voxta.GetCurrentAction(this);
        if (!string.IsNullOrEmpty(newAction))
        {
            ActionItem actionItem = FindActionRoute(newAction);
            if (actionItem == null)
            {
                // Voxta and ALM out of sync
                return;
            }

            if (actionItem.Deferred == false)
            {
                ExecuteAction(actionItem);
                SetChooserToNewActionWhenLinkedToOthers(actionItem);
            }
            else
            {
                deferredActionItem = actionItem;
            }
        }
    }

    private ActionItem FindActionRoute(string action)
    {
        if (lastRoutingAction == null)
        {
            // Voxta and ALM out of sync
            Voxta.SetActionList(this, "");
            return null;
        }
        foreach (string name in lastRoutingAction.LinkedActions)
        {
            ActionItem actionItem = GetItemByName(name);
            if (actionItem.Action == action)
            {
                return actionItem;
            }
        }
        // GLOBAL ACTIONS
        ActionItem globalActionItem = actionsDictionary.Values.FirstOrDefault(actionItem => actionItem.Action == action && actionItem.Layer != null && layerDictionary[actionItem.Layer].Global);
        if (globalActionItem != null) return globalActionItem;

        SuperController.LogError("Did not find route for action = " + action + " in lastAction with name = " + lastRoutingAction.Name);
        return null;
    }

    private void SetChooserToNewActionWhenLinkedToOthers(ActionItem actionItem)
    {
        if (!actionItem.IsEmptyRouting() && IsActionExists(actionItem.Name))
        {
            SetChoosenAction(actionItem.Name);
        }
    }

    public void ExecuteAction(ActionItem actionItem)
    {
        actionItem.Trigger.Trigger();

        if (actionItem.Trigger.HasActionsWithActiveTimers)
        {
            activeTriggers.Add(actionItem.Trigger);
        }

        if (actionItem.PlayTimelineAnim)
        {
            var timelinePlugin = Timeline.GetPlugin(this);
            if (timelinePlugin != null)
            {
                timelinePlugin.CallAction("Play VA_" + actionItem.Action + "/*");
                timelinePlugin.CallAction("Play VA_" + actionItem.Action);
                if (actionItem.TransitionAction)
                {
                    lockedOnTransitionAction = true;
                    lockTransitionAnimationName = "VA_" + actionItem.Action;
                    lastAction = actionItem;
                    SetChooserToNewActionWhenLinkedToOthers(actionItem);
                    UpdateActionList(actionItem);
                    RefreshUI();
                    return;
                }
            }
        }

        UpdateActionList(actionItem);
        UpdateCurrentContext(actionItem);
        lastAction = actionItem;

        if (actionItem.UserAction && !string.IsNullOrEmpty(actionItem.UserTriggerMessage))
        {
            Voxta.TriggerMessage(this, "*" + actionItem.UserTriggerMessage + " and {{char}} says:*");
        }
        RefreshActionUI();
    }

    public void UpdateActionList(ActionItem actionItem)
    {
        if (!actionItem.IsEmptyRouting())
        {
            lastRoutingAction = actionItem;
        }
        if (lastRoutingAction != null)
        {
            var newActionsList = GetActionAndLinkedActionsAsFormattedString();
            if (!string.IsNullOrEmpty(newActionsList) || actionItem.TransitionAction)
            {
                Voxta.SetActionList(this, newActionsList);
            }
            UpdateUserActions();
        }
    }

    public void UpdateCurrentContext(ActionItem actionItem)
    {
        if (!string.IsNullOrEmpty(actionItem.Group))
        {
            var contextFromGroup = groupDictionary[actionItem.Group].Context;

            if (!string.IsNullOrEmpty(contextFromGroup.FreeText))
            {
                currentContext.FreeText = contextFromGroup.FreeText;
            }
            if (!string.IsNullOrEmpty(contextFromGroup.Location))
            {
                currentContext.Location = contextFromGroup.Location;
            }
            if (!string.IsNullOrEmpty(contextFromGroup.Wearing))
            {
                currentContext.Wearing = contextFromGroup.Wearing;
            }
        }

        if (!string.IsNullOrEmpty(actionItem.Context))
        {
            currentContext.FreeText = actionItem.Context;
        }
        if (!string.IsNullOrEmpty(actionItem.Location))
        {
            currentContext.Location = actionItem.Location;
        }
        if (!string.IsNullOrEmpty(actionItem.Wearing))
        {
            currentContext.Wearing = actionItem.Wearing;
        }

        // Build context string for Voxta
        var contextBuilder = new System.Text.StringBuilder();
        if (!string.IsNullOrEmpty(currentContext.FreeText))
        {
            contextBuilder.Append(currentContext.FreeText).Append(". ");
        }
        if (!string.IsNullOrEmpty(currentContext.Location))
        {
            contextBuilder.Append("{{char}} is in ").Append(currentContext.Location).Append(". ");
        }
        if (currentContext.Wearing == "nothing")
        {
            contextBuilder.Append("{{char}} is naked.");
        }
        else if (!string.IsNullOrEmpty(currentContext.Wearing))
        {
            contextBuilder.Append("{{char}} is wearing ").Append(currentContext.Wearing).Append(".");
        }

        //SuperController.LogError("Updating context: " + contextBuilder.ToString());

        Voxta.SetContext(this, contextBuilder.ToString());
        UI.inspectorUI.UpdateCurrentContext(contextBuilder.ToString());
    }

    private void UpdateUserActions()
    {
        List<string> userActions = new List<string>();

        foreach (var linkedActionKey in lastRoutingAction.LinkedActions)
        {
            ActionItem linkedActionItem;
            if (actionsDictionary.TryGetValue(linkedActionKey, out linkedActionItem))
            {
                if (linkedActionItem.UserAction)
                {
                    userActions.Add(linkedActionItem.Name);
                }
            }
        }

        uIAssistProvider.ActionListChanged(userActions);
    }

    private string GetActionAndLinkedActionsAsFormattedString()
    {
        var stringBuilder = new System.Text.StringBuilder();

        foreach (var linkedActionKey in GetLastRoutingPlusGlobalActionNames())
        {
            ActionItem linkedActionItem;
            if (actionsDictionary.TryGetValue(linkedActionKey, out linkedActionItem))
            {
                if (linkedActionItem.UserAction)
                {
                    continue;
                }
                if (lockedOnTransitionAction && !linkedActionItem.IsLayeredAction())
                {
                    continue;
                }

                stringBuilder.AppendLine($"action: {linkedActionItem.Action}");
                stringBuilder.AppendLine($"when: {linkedActionItem.When}");
                if (linkedActionItem.Effect != null && linkedActionItem.Effect != "")
                {
                    stringBuilder.AppendLine($"effect: {linkedActionItem.Effect}");
                }
                if (!string.IsNullOrEmpty(linkedActionItem.Layer))
                {
                    stringBuilder.AppendLine($"layer: {linkedActionItem.Layer}");
                    stringBuilder.AppendLine("finalLayer: false");
                }
                stringBuilder.AppendLine();
            }
        }

        return stringBuilder.ToString().TrimEnd();
    }

    public static JSONStorable getPluginByType(Atom atom, string pluginType, bool optional = false)
    {
        foreach (string pluginId in atom.GetStorableIDs())
        {
            if (pluginId.Contains(pluginType))
            {
                return atom.GetStorableByID(pluginId);
            }
        }

        if (optional)
        {
            return null;
        }

        SuperController.LogError("Plugin with type " + pluginType + " has not been found on the atom " + atom.name);
        throw new NullReferenceException("Cannot find plugin of type " + pluginType + " on atom " + atom.uid);
    }

    public void Update()
    {
        try
        {
            if (lockedOnTransitionAction)
            {
                CheckUnlockTransition();
            }

            ActionItem currentActionItem = GetChoosenItem();
            if (currentActionItem != null && currentActionItem.Trigger.HasActionsWithActiveTimers)
            {
                currentActionItem.Trigger.Update();
            }
            foreach (var trigger in new List<Trigger>(activeTriggers))
            {
                if (trigger.HasActionsWithActiveTimers)
                {
                    trigger.Update();
                }
                else
                {
                    activeTriggers.Remove(trigger);
                }
            }
        }
        catch (Exception e) { SuperController.LogError("Exception caught: " + e); }
    }

    private void CheckUnlockTransition()
    {
        timeSinceLastUnlockCheck += Time.deltaTime;
        if (timeSinceLastUnlockCheck >= 1f)
        {
            bool transitionStillPlaying = false;
            foreach (var action in Timeline.GetPlugin(this).GetStringChooserParamNames())
            {
                if (action.Contains("Animations (") && Timeline.GetPlugin(this).GetStringChooserParamValue(action).Contains(lockTransitionAnimationName))
                {
                    transitionStillPlaying = true;
                }
            }

            if (!transitionStillPlaying)
            {
                OnTransitionActionFinished();
            }

            timeSinceLastUnlockCheck = 0f;
        }
    }

    public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
    {
        base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
        actionsChooser.choices = actionsDictionary.Keys.ToList();
        actionsChooser.valNoCallback = actionsDictionary.Keys.FirstOrDefault();
        RefreshUI();
        UI.groupUI.RefreshUI();
    }

    public void RefreshUI()
    {
        if (!ShouldDeferRefresh())
        {
            Utils.RemoveUIElements(this, mainUI.uiElements.Cast<object>().ToList());
            mainUI.uiElements.Clear();
            if (!lockedOnTransitionAction)
            {
                actionsChooser.choices = actionsDictionary.Keys.ToList();
                actionsChooser.valNoCallback = actionsDictionary.Keys.FirstOrDefault();

                new DropDownWithCreateNewBuilder(mainUI).Parent(this.leftUIContent).Build(actionsChooser, OnAddActionButtonClicked);

                var executeButton = Utils.SetupButton(this, "Test Execute Action", OnExecuteActionButtonClicked, false);
                executeButton.buttonColor = new Color(0.45f, 1f, 0.45f);
                mainUI.uiElements.Add(executeButton);

                mainUI.uiElements.Add(Utils.SetupButton(this, "Duplicate Action", OnDuplicateActionButtonClicked, false));

                var removeButton = Utils.SetupButton(this, "Remove Action", OnRemoveActionButtonClicked, false);
                removeButton.buttonColor = Colors.REMOVE_COLOR;
                mainUI.uiElements.Add(removeButton);

                if (lastRoutingAction != null)
                {
                    SetChooserToNewActionWhenLinkedToOthers(lastRoutingAction);
                }
            }
            else
            {
                mainUI.uiElements.Add(Utils.CreateTitle(this, "Blocked on transition action", false, 24, true, true));
                mainUI.uiElements.Add(Utils.CreateTitle(this, "only layered actions can be executed", false, 24, true, true));
                mainUI.uiElements.Add(Utils.CreateTitle(this, "waiting for animation to finish or", false, 24, true, true));
                mainUI.uiElements.Add(Utils.CreateTitle(this, "TransitionActionFinished trigger to fire", false, 24, true, true));
                var executeButton = Utils.SetupButton(this, "Force Unlock", OnTransitionActionFinished, false);
                executeButton.buttonColor = new Color(0.45f, 1f, 0.45f);
                mainUI.uiElements.Add(executeButton);
            }

            RefreshActionUI();
        }
    }

    private bool ShouldDeferRefresh()
    {
        if (PluginUIEventsListener == null || PluginUIEventsListener.IsEnabled)
        {
            return false;
        }

        scheduleRefreshUI = true;
        return true;
    }

    public void OnDestroy()
    {
        SuperController.singleton.BroadcastMessage("OnActionsProviderDestroyed", this, SendMessageOptions.DontRequireReceiver);
        DestroyImmediate(PluginUIEventsListener);
        PluginUIEventsListener = null;
        Utils.OnDestroyUI();
        UnityEngine.Object.Destroy(UI.newLinkedActionsUI.backDrop);
        UI.inspectorUI.Destroy();
    }

    public Transform GetLeftUIContent()
    {
        return leftUIContent;
    }

    public Transform GetRightUIContent()
    {
        return rightUIContent;
    }

    public override bool ShouldIgnore() => false;
}
