using System.Collections.Generic;
using GPUTools.Common.Scripts.Utils;
using SimpleJSON;
public class ActionItem
{
    public const string BOOTSTRAP_NAME = "!BOOTSTRAP!";
    public string Bundle { private set; get; }
    public string Name;
    public string Action;
    public string Group;
    public string When;
    public string Effect;
    public string Context;

    public string Location;

    public string Wearing;

    public bool Deferred;

    public string Layer;

    public bool TransitionAction;

    public bool UserAction;

    public bool PlayTimelineAnim;

    public EventTrigger Trigger;

    public string UserTriggerMessage;

    public List<string> LinkedActions;

    public ActionItem(string newName, ActionItem actionItem)
    {
        Bundle = "";
        Name = newName;
        Action = ActionNameUtils.FunctionFormatActionName(newName);
        Group = actionItem.Group;
        When = actionItem.When;
        Effect = actionItem.Effect;
        Context = actionItem.Context;
        Location = actionItem.Location;
        Wearing = actionItem.Wearing;
        Deferred = actionItem.Deferred;
        Layer = actionItem.Layer;
        TransitionAction = actionItem.TransitionAction;
        UserAction = actionItem.UserAction;
        PlayTimelineAnim = actionItem.PlayTimelineAnim;
        Trigger = new EventTrigger(actionItem.Trigger);
        UserTriggerMessage = actionItem.UserTriggerMessage;
        LinkedActions = new List<string>(actionItem.LinkedActions);
    }

    public ActionItem(string name, string action, string when, MVRScript script)
    {
        Bundle = "";
        Name = name;
        Action = action;
        When = when;
        Group = "";
        Effect = "";
        Context = "";
        Location = "";
        Wearing = "";
        Layer = "";
        PlayTimelineAnim = true;
        Trigger = new EventTrigger(script, "Trigger");
        UserTriggerMessage = "";
        LinkedActions = new List<string>();
    }

    public ActionItem(JSONClass actionJSON, string bundle, MVRScript script, bool reattatchTriggerReceivers = false) : this(actionJSON, script, reattatchTriggerReceivers)
    {
        this.Bundle = bundle;
    }

    public ActionItem(JSONClass actionJSON, MVRScript script, bool reattatchTriggerReceivers = false)
    {
        Bundle = actionJSON["Bundle"];
        if (Bundle == null)
        {
            Bundle = "";
        }
        Action = actionJSON["Action"];
        Group = actionJSON["Group"];
        if (Group == null)
        {
            Group = "";
        }
        Name = actionJSON["Name"];
        if (Name == null)
        {
            Name = Action;
        }
        Layer = actionJSON["Layer"];
        if (Layer == null)
        {
            Layer = "";
        }
        When = actionJSON["When"];
        Effect = actionJSON["Effect"];
        Context = actionJSON["Context"];
        Location = actionJSON["Location"];
        Wearing = actionJSON["Wearing"];
        Deferred = actionJSON["Deferred"].AsBool;
        TransitionAction = actionJSON["TransitionAction"].AsBool;
        UserAction = actionJSON["UserAction"].AsBool;
        PlayTimelineAnim = actionJSON["PlayTimelineAnim"].AsBool;
        JSONClass triggerJSON = actionJSON["Trigger"].AsObject;

        if (reattatchTriggerReceivers)
        {
            ReattatchTriggerReceivers(script, triggerJSON);
        }

        Trigger = new EventTrigger(script, "Trigger");
        Trigger.RestoreFromJSON(triggerJSON);
        UserTriggerMessage = actionJSON["UserTriggerMessage"];

        JSONArray linkedActionsArray = actionJSON["LinkedActions"].AsArray;

        LinkedActions = new List<string>();
        foreach (JSONNode linkedActionJSON in linkedActionsArray)
        {
            LinkedActions.Add(linkedActionJSON.Value);
        }
    }

    public JSONClass GetJSON(string subScenePrefix)
    {
        JSONClass actionJSON = new JSONClass();
        actionJSON["Bundle"] = Bundle;
        actionJSON["Name"] = Name;
        actionJSON["Action"] = Action;
        actionJSON["Group"] = Group;
        actionJSON["When"] = When;
        actionJSON["Effect"] = Effect;
        actionJSON["Context"] = Context;
        actionJSON["Location"] = Location;
        actionJSON["Wearing"] = Wearing;
        actionJSON["Deferred"].AsBool = Deferred;
        actionJSON["Layer"] = Layer;
        actionJSON["TransitionAction"].AsBool = TransitionAction;
        actionJSON["UserAction"].AsBool = UserAction;
        actionJSON["PlayTimelineAnim"].AsBool = PlayTimelineAnim;
        actionJSON["Trigger"] = Trigger.GetJSON(subScenePrefix);
        actionJSON["UserTriggerMessage"] = UserTriggerMessage;

        JSONArray linkedActionsArray = new JSONArray();
        foreach (string linkedAction in LinkedActions)
        {
            linkedActionsArray.Add("", new JSONData(linkedAction));
        }
        actionJSON["LinkedActions"] = linkedActionsArray;

        return actionJSON;
    }

    public bool IsLayeredAction()
    {
        return !string.IsNullOrEmpty(Layer);
    }

    public bool IsEmptyRouting()
    {
        return CollectionUtils.NullOrEmpty(LinkedActions);
    }

    private void ReattatchTriggerReceivers(MVRScript script, JSONClass triggerJSON)
    {
        MVRPluginManager manager = script.containingAtom.GetStorableByID("PluginManager") as MVRPluginManager;
        var scripts = manager.GetComponentsInChildren<MVRScript>();

        foreach (JSONClass startAction in triggerJSON["startActions"].AsArray)
        {
            string receiver = startAction["receiver"];
            string receiverPluginNumber = receiver.Split('_')[0];
            string receiverPluginName = receiver.Split('_')[1];

            bool receiverFound = false;
            foreach (var value in scripts)
            {
                if (receiver == value.storeId)
                {
                    receiverFound = true;
                }
            }

            if (!receiverFound)
            {
                foreach (var value in scripts)
                {
                    if (value.storeId.Contains(receiverPluginName))
                    {
                        startAction["receiver"] = value.storeId;
                        receiverFound = true;
                    }
                }
            }

            if (!receiverFound)
            {
                SuperController.LogError("Cannot reattach '" + receiverPluginName + "' plugin that is a part of this bundle. Action '" + Name + "' may not work properly. To correct this error delete the mentioned action, add the missing plugin to the Person and reimport the bundle again");
            }
        }
    }

    public void CopyFieldValues(ActionItem sourceActionItem)
    {
        When = sourceActionItem.When;
        Effect = sourceActionItem.Effect;
        Context = sourceActionItem.Context;
        Location = sourceActionItem.Location;
        Wearing = sourceActionItem.Wearing;
        UserTriggerMessage = sourceActionItem.UserTriggerMessage;
    }
}
