using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using UnityEngine;

public class Persistence : MVRScript
{
    public Dictionary<string, ActionItem> actionsDictionary = new Dictionary<string, ActionItem>();

    public Dictionary<string, Group> groupDictionary = new Dictionary<string, Group>();

    public Dictionary<string, Layer> layerDictionary = new Dictionary<string, Layer>();

    public Context currentContext = new Context();

    public ActionItem lastRoutingAction;

    public ActionItem GetItemByName(string name)
    {
        if (actionsDictionary.Count > 0)
        {
            ActionItem actionItem;
            if (actionsDictionary.TryGetValue(name, out actionItem))
            {
                return actionItem;
            }

            string stackTrace = StackTraceUtility.ExtractStackTrace();
            SuperController.LogError("Action not found by name = " + name + " " + stackTrace);
        }
        return null;
    }

    public ActionItem GetChoosenItem()
    {
        if (!string.IsNullOrEmpty(MainUI.GetChosenAction()) && IsActionExists(MainUI.GetChosenAction()))
            return actionsDictionary[MainUI.GetChosenAction()];
        return null;
    }

    public bool IsActionExists(string name)
    {
        return actionsDictionary.ContainsKey(name);
    }

    public string[] GetLinkedActionsArray(string name)
    {
        return actionsDictionary[name].LinkedActions.ToArray();
    }

    public List<string> GetLastRoutingPlusGlobalActionNames()
    {
        List<string> routedActions = new List<string>(); routedActions.AddRange(lastRoutingAction.LinkedActions); routedActions.AddRange(GetGlobalActionNames());
        return routedActions;
    }

    public List<string> GetGlobalActionNames()
    {
        List<string> globalActionNames = new List<string>();
        var globalLayers = layerDictionary.Where(l => l.Value.Global).Select(l => l.Key).ToList();
        foreach (var item in actionsDictionary)
        {
            if (globalLayers.Contains(item.Value.Layer))
            {
                globalActionNames.Add(item.Value.Name);
            }
        }

        return globalActionNames;
    }

    public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
    {
        JSONClass jc = new JSONClass();
        jc["id"] = storeId;
        if (includePhysical || forceStore)
        {
            needsStore = true;

            JSONArray actionsArray = new JSONArray();
            JSONArray groupsArray = new JSONArray();
            JSONArray layersArray = new JSONArray();
            foreach (var kvp in actionsDictionary)
            {
                actionsArray.Add(kvp.Value.GetJSON(base.subScenePrefix));
            }

            foreach (var group in groupDictionary)
            {
                groupsArray.Add(group.Value.GetJSON());
            }

            foreach (var layer in layerDictionary)
            {
                layersArray.Add(layer.Value.GetJSON());
            }

            jc["Actions"] = actionsArray;
            jc["Groups"] = groupsArray;
            jc["Layers"] = layersArray;
            if (currentContext != null)
            {
                jc["CurrentContext"] = currentContext.GetJSON();
            }
            if (lastRoutingAction != null)
            {
                jc["LastRoutingAction"] = lastRoutingAction.GetJSON(base.subScenePrefix);
            }
        }

        return jc;
    }

    public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
    {
        if (!awakecalled)
        {
            Awake();
        }

        if (!base.physicalLocked && restorePhysical)
        {
            JSONArray actionsArray = jc["Actions"].AsArray;
            HashSet<string> groups = new HashSet<string>();
            HashSet<string> layers = new HashSet<string>();
            foreach (JSONClass actionJSON in actionsArray)
            {
                ActionItem actionItem = new ActionItem(actionJSON, this);
                actionsDictionary[actionItem.Name] = actionItem;
                if (!string.IsNullOrEmpty(actionItem.Group))
                {
                    groups.Add(actionItem.Group);
                }
                if (!string.IsNullOrEmpty(actionItem.Layer))
                {
                    layers.Add(actionItem.Layer);
                }
            }

            ConstructGroups(jc, groups);
            ConstructLayers(jc, layers);

            JSONClass contextJSON = jc["CurrentContext"].AsObject;
            if (currentContext != null)
            {
                currentContext = new Context(contextJSON);
            }

            JSONClass lastRoutingActionJSON = jc["LastRoutingAction"].AsObject;
            if (lastRoutingActionJSON != null)
            {
                this.lastRoutingAction = new ActionItem(lastRoutingActionJSON, this);
            }
        }
    }

    public void ConstructGroups(JSONClass jc, HashSet<string> groups)
    {
        JSONArray groupArray = jc["Groups"].AsArray;
        if (groupArray == null || groupArray.Count == 0)
        {
            foreach (var group in groups)
            {
                groupDictionary[group] = new Group(group);
            }
        }
        else
        {
            foreach (JSONClass groupJSON in groupArray)
            {
                Group group = new Group(groupJSON);
                groupDictionary[group.Name] = group;
            }
        }
    }

    public void ConstructLayers(JSONClass jc, HashSet<string> layers)
    {
        JSONArray layersArray = jc["Layers"].AsArray;
        if (layersArray == null || layersArray.Count == 0)
        {
            foreach (var layer in layers)
            {
                layerDictionary[layer] = new Layer(layer);
            }
        }
        else
        {
            foreach (JSONClass layerJSON in layersArray)
            {
                Layer layer = new Layer(layerJSON);
                layerDictionary[layer.Name] = layer;
            }
        }
    }

    public override bool ShouldIgnore() => true;
}
