using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using MVR.FileManagementSecure;
using SimpleJSON;

public class SaveManager
{
    private const string SAVE_FOLDER = "Saves\\PluginData\\ALM";

    private static void SavePluginDataJSON(ActionListManager script, string fileName)
    {
        if (string.IsNullOrEmpty(fileName)) return;
        JSONClass jc = script.GetJSON();

        script.SaveJSON(jc, fileName + ".json");
        SuperController.singleton.DoSaveScreenshot(fileName + ".jpg");
    }

    private static void LoadPluginDataJSON(ActionListManager script, string fileName)
    {
        if (string.IsNullOrEmpty(fileName)) return;
        JSONClass jc = script.LoadJSON(fileName).AsObject;
        if (jc == null)
        {
            SuperController.LogMessage("Failed to load action bundle data");
            return;
        }

        JSONArray actionsArray = jc["Actions"].AsArray;
        string startAnim = jc["StartAnim"];
        string bundleName = ExtractBundleName(fileName);
        HashSet<string> groups = new HashSet<string>();
        HashSet<string> layers = new HashSet<string>();


        VerifyTimeline(script, startAnim, actionsArray);
        VerifyVoxtaNeeded(script, actionsArray);

        ActionItem bootstrapAction = null;
        foreach (JSONClass actionJSON in actionsArray)
        {
            ActionItem actionItem = new ActionItem(actionJSON, bundleName, script, true);
            if (script.actionsDictionary.ContainsKey(actionItem.Name))
            {
                SuperController.LogMessage("Current config already contains action '" + actionItem.Name + "' so it won't be imported");
                continue;
            }

            if (actionItem.Name == ActionItem.BOOTSTRAP_NAME)
            {
                bootstrapAction = actionItem;
            }
            else
            {
                script.actionsDictionary.Add(actionItem.Name, actionItem);
                if (!string.IsNullOrEmpty(actionItem.Group))
                {
                    groups.Add(actionItem.Group);
                }
                if (!string.IsNullOrEmpty(actionItem.Layer))
                {
                    layers.Add(actionItem.Layer);
                }
            }

            script.ConstructGroups(jc, groups);
            script.ConstructLayers(jc, layers);
        }

        if (!string.IsNullOrEmpty(startAnim))
        {
            Timeline.GetPlugin(script).CallAction("Play " + startAnim);
        }

        script.RefreshUI();

        if (bootstrapAction != null)
        {
            script.UpdateActionList(bootstrapAction);
            script.UpdateCurrentContext(bootstrapAction);
            script.lastRoutingAction = bootstrapAction;
            UI.inspectorUI.RefreshUI();
        }
    }

    private static void VerifyVoxtaNeeded(MVRScript script, JSONArray actionsArray)
    {
        foreach (JSONClass actionJSON in actionsArray)
        {
            if (new ActionItem(actionJSON, script).Name == "!BOOTSTRAP!" && Voxta.GetPlugin(script, true) == null)
            {
                SuperController.LogError("Voxta must be added to this atom in order to import this action bundle");
                throw new NullReferenceException();
            }
        }
    }

    private static void VerifyTimeline(MVRScript script, string startAnim, JSONArray actionsArray)
    {
        JSONStorable timeline = Timeline.GetPlugin(script);
        if (!string.IsNullOrEmpty(startAnim) && timeline == null)
        {
            SuperController.LogError("Timeline must be added to this atom in order to import this action bundle");
            throw new NullReferenceException();
        }

        foreach (JSONClass actionJSON in actionsArray)
        {
            if (new ActionItem(actionJSON, script).PlayTimelineAnim && timeline == null)
            {
                SuperController.LogError("Timeline must be added to this atom in order to import this action bundle");
                throw new NullReferenceException();
            }
        }

        if (!string.IsNullOrEmpty(startAnim) && !timeline.GetActionNames().Contains("Play " + startAnim))
        {
            SuperController.LogError("Cannot find triggers in Timeline. It's a bug in Timelinie, please change one of the animation names and then change it back to the original value. Then import the bundle again.");
            throw new NullReferenceException();
        }
    }

    public static void Export(ActionListManager script)
    {
        try
        {
            FileManagerSecure.CreateDirectory(SAVE_FOLDER);
            var fileBrowserUI = SuperController.singleton.fileBrowserUI;
            fileBrowserUI.SetTitle("Export action bundle");
            fileBrowserUI.fileRemovePrefix = null;
            fileBrowserUI.hideExtension = false;
            fileBrowserUI.keepOpen = false;
            fileBrowserUI.fileFormat = "json";
            fileBrowserUI.defaultPath = SAVE_FOLDER;
            fileBrowserUI.showDirs = true;
            fileBrowserUI.shortCuts = null;
            fileBrowserUI.browseVarFilesAsDirectories = false;
            fileBrowserUI.SetTextEntry(true);
            fileBrowserUI.Show((fileName) => SavePluginDataJSON(script, fileName));
            fileBrowserUI.ActivateFileNameField();
        }
        catch (Exception exc)
        {
            SuperController.LogError($"Failed to save file dialog: {exc}");
        }
    }

    public static void Import(ActionListManager script)
    {
        try
        {
            FileManagerSecure.CreateDirectory(SAVE_FOLDER);
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(SAVE_FOLDER);
            SuperController.singleton.GetMediaPathDialog((fileName) => LoadPluginDataJSON(script, fileName), "json", SAVE_FOLDER, false, true, false, null, false, shortcuts);
        }
        catch (Exception exc)
        {
            SuperController.LogError($"Timeline: Failed to open file dialog: {exc}");
        }
    }

    private static string ExtractBundleName(string fileName)
    {
        var match = Regex.Match(fileName, @"vaan20\.(\w+)ActionBundle\.(\d+)");
        if (match.Success)
        {
            return $"{match.Groups[1].Value}.v{match.Groups[2].Value}";
        }

        SuperController.LogError("Bundle name format not matched");

        return "bundleFault";
        throw new NullReferenceException();
    }

}
