using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using System;
using UnityEngine;
using static MainUI;
using Battlehub.UIControls;

public class ActionTextFieldsUI : UIPart
{
    private Dictionary<string, ActionItem> actionsDictionary;
    private JSONStorableString nameField;
    private JSONStorableString actionField;
    private JSONStorableString groupField;
    public JSONStorableString whenField;
    private JSONStorableString effectField;
    private JSONStorableString locationField;
    private JSONStorableString wearingField;
    private JSONStorableString contextField;
    private UIDynamicButton triggerButton;
    private JSONStorableString userTriggerMessageField;

    public ActionTextFieldsUI(ActionListManager script) : base(script)
    {
        actionsDictionary = script.actionsDictionary;
    }

    public override void Init()
    {
        nameField = new JSONStorableString("Name", "", OnNameFieldChanged);
        actionField = new JSONStorableString("Action", "", OnActionFieldChanged);
        groupField = new JSONStorableString("Group", "", OnGroupFieldChanged);
        whenField = new JSONStorableString("When", "", OnWhenFieldChanged);
        effectField = new JSONStorableString("Effect", "", OnEffectFieldChanged);
        locationField = new JSONStorableString("Location", "", OnLocationFieldChanged);
        wearingField = new JSONStorableString("Wearing", "", OnWearingFieldChanged);
        contextField = new JSONStorableString("Context", "", OnContextFieldChanged);
        userTriggerMessageField = new JSONStorableString("UserTriggerMessage", "", OnUserTriggerMessageChanged);
    }

    override
    public void RefreshUI()
    {
        RemoveAllUI();
        if (!script.lockedOnTransitionAction)
        {
            DrawUI();
        }
    }

    public void resetAllFields()
    {
        nameField.val = "";
        actionField.val = "";
        whenField.val = "";
        effectField.val = "";
        locationField.val = "";
        wearingField.val = "";
        contextField.val = "";
        userTriggerMessageField.val = "";
    }

    public void copyFieldValues(ActionItem actionItem)
    {
        if (actionItem != null && nameField != null)
        {
            nameField.valNoCallback = actionItem.Name;
            actionField.val = actionItem.Action;
            groupField.val = actionItem.Group;
            whenField.val = actionItem.When;
            effectField.val = actionItem.Effect;
            locationField.val = actionItem.Location;
            wearingField.val = actionItem.Wearing;
            contextField.val = actionItem.Context;
            userTriggerMessageField.val = actionItem.UserTriggerMessage;
        }
    }
    private void DrawUI()
    {
        new TextInputBuilder(script, uiElements, "Name", nameField).Build();

        if (!string.IsNullOrEmpty(GetChosenAction())
        && (ActionNameUtils.FunctionFormatActionName(script.GetChoosenItem().Name) != script.GetChoosenItem().Action
        || !UI.linkedActionsUI.sameActionNameFunction.val))
        {
            new TextInputBuilder(script, uiElements, "Function", actionField).Build();
        }

        JSONStorableStringChooser groupsChooser = new JSONStorableStringChooser("Group", new List<string> { "" }, script.GetChoosenItem().Group, "Group", OnGroupChanged);
        groupsChooser.choices.AddRange(script.groupDictionary.Keys.ToList());
        new DropDownWithCreateNewBuilder(this).Parent(script.GetLeftUIContent()).Build(groupsChooser, () => { UI.groupUI.Open(script.GetChoosenItem().Group); });

        if (!script.GetChoosenItem().UserAction)
        {
            new TitleBuilder(this, "When").Alignment(TextAnchor.MiddleCenter).FontSize(26).FontBold().Build();
            uiElements.Add(Utils.CreateMultilineTextInput(script, whenField, false, false, 100f));
        }
        new TitleBuilder(this, "Effect").Alignment(TextAnchor.MiddleCenter).FontSize(26).FontBold().Build();
        uiElements.Add(Utils.CreateMultilineTextInput(script, effectField, false, false, 70f));
        new TitleBuilder(this, "Context").Alignment(TextAnchor.MiddleCenter).FontSize(26).FontBold().Build();
        new TextInputBuilder(script, uiElements, "Location", locationField).Build();
        new TextInputBuilder(script, uiElements, "Wearing", wearingField).Build();
        uiElements.Add(Utils.CreateMultilineTextInput(script, contextField, false, false, 100f));

        if (script.GetChoosenItem().UserAction)
        {
            new TitleBuilder(this, "Trigger Message").Alignment(TextAnchor.MiddleCenter).FontSize(26).FontBold().Build();
            uiElements.Add(Utils.CreateMultilineTextInput(script, userTriggerMessageField, false, false, 90f));
        }

        string actionKey = GetChosenAction();
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            triggerButton = Utils.SetupButton(script, "Action " + actionsDictionary[actionKey].Trigger.Name, actionsDictionary[actionKey].Trigger.OpenPanel, false);
            triggerButton.buttonColor = new Color(0.9f, 0.9f, 0.2f);
            uiElements.Add(triggerButton);
        }

        SimpleTriggerHandler.LoadAssets();

        if (!string.IsNullOrEmpty(GetChosenAction()))
        {
            copyFieldValues(script.GetChoosenItem());
        }
        else
        {
            resetAllFields();
        }
    }

    private void OnLayerChanged(string val)
    {
        throw new NotImplementedException();
    }

    private void OnGroupChanged(string val)
    {
        OnGroupFieldChanged(val);
    }

    private void OnActionFieldChanged(string newActionName)
    {
        string actionKey = GetChosenAction();
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            ActionItem actionItemToChange = script.GetItemByName(actionKey);
            ActionItem existing = CheckForActionDuplicates(actionItemToChange, newActionName);
            if (existing != null)
            {
                SuperController.LogError("Function name change would cause duplicate route on action = " + existing.Name);
                actionField.val = actionsDictionary[actionKey].Action;
                return;
            }
            actionsDictionary[actionKey].Action = newActionName;
        }
    }

    public ActionItem CheckForActionDuplicates(ActionItem actionItemToChange, string newActionValue)
    {
        foreach (var actionPair in actionsDictionary)
        {
            var actionItem = actionPair.Value;
            if (actionItem.LinkedActions.Count > 0)
            {
                var linkedActionsValues = actionItem.LinkedActions
                    .Select(name => actionsDictionary.ContainsKey(name) ? actionsDictionary[name].Action : null)
                    .ToList();

                if (linkedActionsValues.Contains(actionItemToChange.Action))
                {
                    linkedActionsValues[linkedActionsValues.IndexOf(actionItemToChange.Action)] = newActionValue;
                }

                if (linkedActionsValues.GroupBy(x => x).Any(g => g.Count() > 1))
                {
                    return actionItem;
                }
            }
        }

        return null;
    }

    public void OnNameFieldChanged(string newName)
    {
        string oldName = GetChosenAction();
        if (string.IsNullOrEmpty(oldName) || oldName == newName)
        {
            return;
        }

        if (actionsDictionary.ContainsKey(newName))
        {
            SuperController.LogError("Action with the new name already exists.");
            nameField.val = oldName;
            return;
        }

        ActionItem actionItem;
        if (actionsDictionary.TryGetValue(oldName, out actionItem))
        {
            actionsDictionary.Remove(oldName);

            if (ActionNameUtils.FunctionFormatActionName(actionItem.Name) == actionItem.Action)
            {
                actionItem.Action = ActionNameUtils.FunctionFormatActionName(newName);
            }
            actionItem.Name = newName;

            actionsDictionary[newName] = actionItem;

            foreach (var item in actionsDictionary.Values.ToArray())
            {
                if (item.LinkedActions.Contains(oldName))
                {
                    item.LinkedActions.Remove(oldName);
                    item.LinkedActions.Add(newName);
                }
            }

            actionsChooser.choices = actionsDictionary.Keys.ToList();
            actionsChooser.valNoCallback = newName;

            UI.linkedActionsUI.RefreshUI();
            UI.inspectorUI.RefreshUI();
            UI.bulkActionManagerUI.RefreshUI();

            script.uIAssistProvider.ChangeName(oldName, newName);
        }
    }


    private void OnGroupFieldChanged(string newValue)
    {
        var item = script.GetChoosenItem();
        if (item != null)
            item.Group = newValue;
        UI.linkedActionsUI.RefreshUI();
        UI.bulkActionManagerUI.RefreshUI();
        RefreshUI();
    }

    private void OnContextFieldChanged(string newValue)
    {
        var item = script.GetChoosenItem();
        if (item != null)
        {
            item.Context = newValue.TrimEnd('\r', '\n');
            contextField.valNoCallback = item.Context;
        }
    }
    private void OnWhenFieldChanged(string newValue)
    {
        var item = script.GetChoosenItem();
        if (item != null)
        {
            item.When = newValue.TrimEnd('\r', '\n');
            whenField.valNoCallback = item.When;
        }
    }

    private void OnEffectFieldChanged(string newValue)
    {
        var item = script.GetChoosenItem();
        if (item != null)
        {
            item.Effect = newValue.TrimEnd('\r', '\n');
            effectField.valNoCallback = item.Effect;
        }
    }

    private void OnLocationFieldChanged(string newValue)
    {
        var item = script.GetChoosenItem();
        if (item != null)
            item.Location = newValue;
    }

    private void OnWearingFieldChanged(string newValue)
    {
        var item = script.GetChoosenItem();
        if (item != null)
            item.Wearing = newValue;
    }

    private void OnUserTriggerMessageChanged(string newValue)
    {
        var item = script.GetChoosenItem();
        if (item != null)
            item.UserTriggerMessage = newValue;
    }


}
