using System;
using System.Collections.Generic;
using System.Linq;
using DynamicCSharp;
using UnityEngine;

public class GroupUI : UIPart
{
    private GameObject backDrop;
    private JSONStorableStringChooser groupsChooser;
    private RectTransform scrollContentLeft;
    private int newGroupCounter = 1;
    private JSONStorableString nameField;
    private JSONStorableString locationField;
    private JSONStorableString wearingField;
    private JSONStorableString freeTextField;

    public GroupUI(ActionListManager script) : base(script)
    {
    }

    public void ToggleOpen()
    {
        backDrop.SetActive(!backDrop.activeSelf);
    }

    public void Open(string groupName)
    {
        if (groupsChooser.choices.Contains(groupName))
        {
            groupsChooser.val = groupName;
        }
        backDrop.SetActive(true);
    }

    public void Close()
    {
        backDrop.SetActive(false);
    }

    private void InitUI()
    {
        backDrop = DrawBackdrop("GroupUI", script.UITransform, new Vector2(250, 340), new Vector2(-250, -340), Colors.MAIN_WINDOW_BACK, 2);
        scrollContentLeft = CreateScroll(backDrop.transform, new Vector2(0.05f, 0.05f), new Vector2(0.95f, 0.95f), 15);

        RefreshUI();
    }

    public override void Init()
    {
        groupsChooser = new JSONStorableStringChooser("Group", script.groupDictionary.Keys.ToList(), "", "Group", OnGroupChanged);
        nameField = new JSONStorableString("Name", "", OnNameFieldChanged);
        locationField = new JSONStorableString("Location", "", OnLocationFieldChanged);
        wearingField = new JSONStorableString("Wearing", "", OnWearingFieldChanged);
        freeTextField = new JSONStorableString("FreeText", "", OnFreeTextFieldChanged);

        InitUI();
        backDrop.SetActive(false);
    }

    private void OnGroupChanged(string val)
    {
        RefreshUI();
    }

    private void OnFreeTextFieldChanged(string val)
    {
        var group = GetChoosenGroup();
        if (group != null)
            group.Context.FreeText = val;
        RefreshUI();
    }

    private void OnWearingFieldChanged(string val)
    {
        var group = GetChoosenGroup();
        if (group != null)
            group.Context.Wearing = val;
        RefreshUI();
    }

    private void OnLocationFieldChanged(string val)
    {
        var group = GetChoosenGroup();
        if (group != null)
            group.Context.Location = val;
        RefreshUI();
    }

    private void OnNameFieldChanged(string newName)
    {
        string oldName = GetChoosenGroup().Name;
        if (string.IsNullOrEmpty(oldName) || oldName == newName)
        {
            return;
        }

        if (script.groupDictionary.ContainsKey(newName))
        {
            SuperController.LogError("Group with the new name already exists.");
            nameField.val = oldName;
            return;
        }

        Group group;
        if (script.groupDictionary.TryGetValue(oldName, out group))
        {
            script.groupDictionary.Remove(oldName);

            group.Name = newName;

            script.groupDictionary[newName] = group;

            groupsChooser.choices = script.groupDictionary.Keys.ToList();
            groupsChooser.valNoCallback = newName;

            foreach (var actionItem in script.actionsDictionary.Values.ToList())
            {
                if (actionItem.Group == oldName)
                {
                    actionItem.Group = newName;
                }
            }

            RefreshUI();
            UI.actionTextFieldsUI.RefreshUI();
            UI.bulkActionManagerUI.RefreshUI();
        }
    }

    public override void RefreshUI()
    {
        RemoveAllUI();

        groupsChooser.choices = script.groupDictionary.Keys.ToList();
        if (GetChoosenGroup() == null)
        {
            groupsChooser.valNoCallback = script.groupDictionary.Keys.FirstOrDefault();
        }

        new DropDownWithCreateNewBuilder(this).Parent(scrollContentLeft.transform).Build(groupsChooser, OnAddGroupClicked);

        var removeButton = Utils.SetupButton(script, "Remove Group", OnRemoveGroupButtonClicked, false);
        removeButton.transform.SetParent(scrollContentLeft);
        removeButton.buttonColor = Colors.REMOVE_COLOR;
        uiElements.Add(removeButton);

        new TextInputBuilder(script, uiElements, "Name", nameField).Parent(scrollContentLeft).Build();

        new TitleBuilder(this, "Context").Parent(scrollContentLeft).Alignment(TextAnchor.MiddleCenter).FontSize(26).FontBold().Build();
        new TextInputBuilder(script, uiElements, "Location", locationField).Parent(scrollContentLeft).Build();
        new TextInputBuilder(script, uiElements, "Wearing", wearingField).Parent(scrollContentLeft).Build();
        UIDynamic freeText = Utils.CreateMultilineTextInput(script, freeTextField, false, false, 100f);
        freeText.transform.SetParent(scrollContentLeft);
        uiElements.Add(freeText);


        if (!string.IsNullOrEmpty(groupsChooser.val))
        {
            CopyFieldValues(GetChoosenGroup());
        }
        else
        {
            ResetAllFields();
        }

        var spacer = script.CreateSpacer();
        spacer.transform.SetParent(scrollContentLeft);
        spacer.height = 8;
        uiElements.Add(spacer);
        new ButtonBuilder(this, "Close", Close).Parent(scrollContentLeft).Build();
    }

    private void OnAddGroupClicked()
    {
        string nameValue = "new group " + newGroupCounter++;

        Group newGroup = new Group(nameValue);

        script.groupDictionary.Add(nameValue, newGroup);
        groupsChooser.choices = script.groupDictionary.Keys.ToList();
        groupsChooser.val = nameValue;

        RefreshUI();
        UI.actionTextFieldsUI.RefreshUI();
        UI.bulkActionManagerUI.RefreshUI();
    }

    public void ResetAllFields()
    {
        nameField.valNoCallback = "";
        locationField.valNoCallback = "";
        wearingField.valNoCallback = "";
        freeTextField.valNoCallback = "";
    }

    public void CopyFieldValues(Group group)
    {
        if (group != null && nameField != null)
        {
            nameField.valNoCallback = group.Name;
            locationField.val = group.Context.Location;
            wearingField.val = group.Context.Wearing;
            freeTextField.val = group.Context.FreeText;
        }
    }

    public Group GetChoosenGroup()
    {
        if (!string.IsNullOrEmpty(groupsChooser.val) && script.groupDictionary.ContainsKey(groupsChooser.val))
            return script.groupDictionary[groupsChooser.val];
        return null;
    }

    private void OnRemoveGroupButtonClicked()
    {
        string groupToRemoveName = GetChoosenGroup().Name;
        if (string.IsNullOrEmpty(groupToRemoveName))
        {
            return;
        }

        if (script.groupDictionary.ContainsKey(groupToRemoveName))
        {
            foreach (var actionItem in script.actionsDictionary.Values.ToList())
            {
                if (actionItem.Group == groupToRemoveName)
                {
                    actionItem.Group = "";
                }
            }

            int indexToRemove = groupsChooser.choices.IndexOf(groupToRemoveName);
            int nextIndex = 0;
            if (indexToRemove >= 0 && indexToRemove < groupsChooser.choices.Count - 1)
            {
                nextIndex = indexToRemove;
            }

            script.groupDictionary.Remove(groupToRemoveName);

            groupsChooser.choices = script.groupDictionary.Keys.ToList();

            if (groupsChooser.choices.Count > 0)
            {
                groupsChooser.val = groupsChooser.choices[nextIndex];
            }

            UI.bulkActionManagerUI.RefreshUI();
            RefreshUI();
        }
        else
        {
            SuperController.LogError("Group to remove not found in dictionary.");
        }
    }
}
