using System;
using System.Collections.Generic;
using System.Linq;
using DynamicCSharp;
using UnityEngine;
using UnityEngine.UI;

public class LayerUI : UIPart
{
    private GameObject backDropLock;
    private GameObject backDrop;
    private JSONStorableStringChooser layerChooser;
    private RectTransform scrollContent;
    private int newLayerCounter = 1;
    private JSONStorableString nameField;
    private JSONStorableBool globalField;

    public LayerUI(ActionListManager script) : base(script)
    {
    }

    public void ToggleOpen()
    {
        backDrop.SetActive(!backDrop.activeSelf);
    }

    public void Open(string layerName)
    {
        if (layerChooser.choices.Contains(layerName))
        {
            layerChooser.val = layerName;
        }
        backDrop.SetActive(true);
    }

    public void Close()
    {
        backDrop.SetActive(false);
    }

    private void InitUI()
    {
        // backDropLock = new GameObject("Backdrop Lock");
        // backDropLock.transform.SetParent(script.UITransform, false);

        // var rectLock = backDropLock.AddComponent<RectTransform>();
        // rectLock.anchorMin = Vector2.zero;
        // rectLock.anchorMax = Vector2.one;
        // rectLock.offsetMin = new Vector2(0, 0);
        // rectLock.offsetMax = new Vector2(0, 0);

        // var bgRect = backDropLock.AddComponent<Image>();

        // bgRect.color = new Color(0, 0, 0, 0.7f);
        backDrop = DrawBackdrop("LayerUI", script.UITransform, new Vector2(250, 400), new Vector2(-250, -400), Colors.MAIN_WINDOW_BACK, 2);
        scrollContent = CreateScroll(backDrop.transform, new Vector2(0.05f, 0.05f), new Vector2(0.95f, 0.95f), 15);

        RefreshUI();
    }

    public override void Init()
    {
        layerChooser = new JSONStorableStringChooser("Layer", script.layerDictionary.Keys.ToList(), "", "Layer", OnLayerChanged);
        nameField = new JSONStorableString("Name", "", OnNameFieldChanged);
        globalField = new JSONStorableBool("Global", false, OnGlobalFieldChanged);

        InitUI();
        backDrop.SetActive(false);
    }

    private void OnGlobalFieldChanged(bool val)
    {
        var layer = GetChoosenLayer();
        if (layer != null)
        {
            if (layer.GetActions(script).GroupBy(ai => ai.Action).Any(g => g.Count() > 1))
            {
                SuperController.LogError("Layer cannot be set to global as it has actions with the same function name");
                RefreshUI();
                return;
            }
            layer.Global = val;
            RefreshUI();
        }
    }

    private void OnLayerChanged(string val)
    {
        RefreshUI();
    }

    private void OnNameFieldChanged(string newName)
    {
        string oldName = GetChoosenLayer().Name;
        if (string.IsNullOrEmpty(oldName) || oldName == newName)
        {
            return;
        }

        if (script.layerDictionary.ContainsKey(newName))
        {
            SuperController.LogError("Layer with the new name already exists.");
            nameField.val = oldName;
            return;
        }

        Layer layer;
        if (script.layerDictionary.TryGetValue(oldName, out layer))
        {
            script.layerDictionary.Remove(oldName);

            layer.Name = newName;

            script.layerDictionary[newName] = layer;

            layerChooser.choices = script.layerDictionary.Keys.ToList();
            layerChooser.valNoCallback = newName;

            foreach (var actionItem in script.actionsDictionary.Values.ToList())
            {
                if (actionItem.Layer == oldName)
                {
                    actionItem.Layer = newName;
                }
            }

            RefreshUI();
            UI.actionTextFieldsUI.RefreshUI();
            UI.bulkActionManagerUI.RefreshUI();
            UI.linkedActionsUI.RefreshUI();
        }
    }

    public override void RefreshUI()
    {
        RemoveAllUI();

        layerChooser.choices = script.layerDictionary.Keys.ToList();
        if (GetChoosenLayer() == null)
        {
            layerChooser.valNoCallback = script.layerDictionary.Keys.FirstOrDefault();
        }

        new DropDownWithCreateNewBuilder(this).Parent(scrollContent.transform).Build(layerChooser, OnAddLayerClicked);

        var removeButton = Utils.SetupButton(script, "Remove Layer", OnRemoveLayerButtonClicked, false);
        removeButton.transform.SetParent(scrollContent);
        removeButton.buttonColor = Colors.REMOVE_COLOR;
        uiElements.Add(removeButton);

        new TextInputBuilder(script, uiElements, "Name", nameField).Parent(scrollContent).Build();

        var globalLabel1 = Utils.CreateTitle(script, "Actions in the global layer will be routed", true, 24, true, false);
        var globalLabel2 = Utils.CreateTitle(script, "on every action inference automatically", true, 24, true, false);
        globalLabel1.transform.SetParent(scrollContent);
        globalLabel2.transform.SetParent(scrollContent);
        uiElements.Add(globalLabel1);
        uiElements.Add(globalLabel2);

        var globalToggle = Utils.SetupToggle(script, globalField, true);
        globalToggle.transform.SetParent(scrollContent);
        uiElements.Add(globalToggle);

        if (!string.IsNullOrEmpty(layerChooser.val))
        {
            CopyFieldValues(GetChoosenLayer());
        }
        else
        {
            ResetAllFields();
        }

        var spacer = script.CreateSpacer();
        spacer.transform.SetParent(scrollContent);
        spacer.height = 8;
        uiElements.Add(spacer);
        new ButtonBuilder(this, "Close", Close).Parent(scrollContent).Build();
    }

    private void OnAddLayerClicked()
    {
        string nameValue = "new layer " + newLayerCounter++;

        Layer newLayer = new Layer(nameValue);

        script.layerDictionary.Add(nameValue, newLayer);
        layerChooser.choices = script.layerDictionary.Keys.ToList();
        layerChooser.val = nameValue;

        RefreshUI();
        UI.actionTextFieldsUI.RefreshUI();
        UI.bulkActionManagerUI.RefreshUI();
        UI.linkedActionsUI.RefreshUI();
    }

    public void ResetAllFields()
    {
        nameField.valNoCallback = "";
        globalField.valNoCallback = false;
    }

    public void CopyFieldValues(Layer layer)
    {
        if (layer != null && nameField != null)
        {
            nameField.valNoCallback = layer.Name;
            globalField.valNoCallback = layer.Global;
        }
    }

    public Layer GetChoosenLayer()
    {
        if (!string.IsNullOrEmpty(layerChooser.val) && script.layerDictionary.ContainsKey(layerChooser.val))
            return script.layerDictionary[layerChooser.val];
        return null;
    }

    private void OnRemoveLayerButtonClicked()
    {
        string layerToRemoveName = GetChoosenLayer().Name;
        if (string.IsNullOrEmpty(layerToRemoveName))
        {
            return;
        }

        if (script.layerDictionary.ContainsKey(layerToRemoveName))
        {
            foreach (var actionItem in script.actionsDictionary.Values.ToList())
            {
                if (actionItem.Layer == layerToRemoveName)
                {
                    actionItem.Layer = "";
                }
            }

            int indexToRemove = layerChooser.choices.IndexOf(layerToRemoveName);
            int nextIndex = 0;
            if (indexToRemove >= 0 && indexToRemove < layerChooser.choices.Count - 1)
            {
                nextIndex = indexToRemove;
            }

            script.layerDictionary.Remove(layerToRemoveName);

            layerChooser.choices = script.layerDictionary.Keys.ToList();

            if (layerChooser.choices.Count > 0)
            {
                layerChooser.val = layerChooser.choices[nextIndex];
            }

            UI.bulkActionManagerUI.RefreshUI();
            UI.linkedActionsUI.RefreshUI();
            RefreshUI();
        }
        else
        {
            SuperController.LogError("Layer to remove not found in dictionary.");
        }
    }
}
