using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;
using static MainUI;

public class NewLinkedActionsUI : UIExtention
{
    private bool isOpen;
    private Dictionary<string, ActionItem> actionsDictionary;
    public GameObject backDropLock;
    public RectTransform scrollContent;
    private RectTransform scrollContentLeft;
    private RectTransform scrollContentRight;
    private Dictionary<string, bool> groupExpanded = new Dictionary<string, bool>();
    private float lastClickTime = 0f;
    private const float doubleClickThreshold = 0.2f;
    private string pendingSingleClickName;
    private JSONStorableString searchField;
    private UIDynamicButton closeButton;

    public NewLinkedActionsUI(ActionListManager script) : base(script)
    {
        actionsDictionary = script.actionsDictionary;

    }

    public override void Init()
    {
        isOpen = true;

        Init(LinkActionsCoordinates.GET_BACKDROP_OFFSET_MIN(), LinkActionsCoordinates.GET_BACKDROP_OFFSET_MAX());
        searchField = new JSONStorableString("Search", "", OnSearchChanged);

        scrollContentLeft = CreateScroll(backDrop.transform, new Vector2(0.01f, 0.05f), new Vector2(0.49f, 0.95f));
        scrollContentRight = CreateScroll(backDrop.transform, new Vector2(0.51f, 0.05f), new Vector2(0.99f, 0.95f));

        closeButton = UIHelpers.CreateCustomButton(script, "R<", LinkActionsCoordinates.GET_FOLD_LOCATION(), new Vector2(-1040, 690));
        closeButton.button.onClick.AddListener(() =>
        {
            backDrop.gameObject.SetActive(!backDrop.gameObject.activeSelf);
            closeButton.label = (isOpen) ? "R>" : "R<";
            if (!isOpen)
            {
                UI.bulkActionManagerUI.CloseBackdrop();
            }
            isOpen = !isOpen;
        });

        new TitleBuilder(this, "Action Routing").FontSize(38).FontBold().Parent(backDrop.transform).Location(new Vector2(20, -76)).Size(new Vector2(0, 100)).ConstantElement().Build();

        new TextInputBuilder(script, uiElements, "Search:", searchField)
        .Parent(backDrop.transform).Location(new Vector2(620, -45)).Size(new Vector2(500, 40)).ConstantElement().Build();

        new ButtonBuilder(this, "X", () => { searchField.val = ""; })
        .Parent(backDrop.transform).Location(new Vector2(1125, -55)).Size(new Vector2(-1130, 50)).ConstantElement().Build();

        new ButtonBuilder(this, "+", () => ExpandAllGroups()).Parent(backDrop.transform).Location(new Vector2(500, -55)).Size(new Vector2(-1130, 50)).ConstantElement().Build();
        new ButtonBuilder(this, "-", () => CloseAllGroups()).Parent(backDrop.transform).Location(new Vector2(550, -55)).Size(new Vector2(-1130, 50)).ConstantElement().Build();

        new TitleBuilder(this, "Single click - add/remove link. Double click - execute action.\nWhite - normal actions. Yellow - layered actions. Green - transition actions. Blue - user actions")
        .FontSize(22).LabelColor(Color.white).Parent(backDrop.transform).Location(new Vector2(10, -1390)).Size(new Vector2(5, 100)).ConstantElement().Build();
    }

    public void CloseBackdrop()
    {
        isOpen = false;
        backDrop.gameObject.SetActive(false);
        closeButton.label = "R>";
    }

    override
    public void RefreshUI()
    {
        RemoveAllUI();
        UnityEngine.Object.Destroy(backDropLock);

        closeButton.label = (isOpen) ? "R<" : "R>";

        if (actionsDictionary.Count > 1)
        {
            DrawRemainingActions();
            DrawLinkedActions();
        }

        if (script.lockedOnTransitionAction)
        {
            backDropLock = new GameObject("Backdrop Lock");
            backDropLock.transform.SetParent(backDrop.transform, false);

            var rectLock = backDropLock.AddComponent<RectTransform>();
            rectLock.anchorMin = Vector2.zero;
            rectLock.anchorMax = Vector2.one;
            rectLock.offsetMin = new Vector2(0, 0);
            rectLock.offsetMax = new Vector2(0, 0);

            var bgRect = backDropLock.AddComponent<Image>();
            bgRect.color = new Color(0, 0, 0, 0.7f);
        }
    }

    // GROUP HANDLING
    private List<IGrouping<string, ActionItem>> GroupActions()
    {
        return actionsDictionary.Values
                .GroupBy(a => a.Group)
                .OrderBy(g => g.Key)
                .ToList();
    }

    private void DrawRemainingActions()
    {
        new TitleBuilder(this, "╔═══════  Remaining Actions  ═══════╗").LabelColor(Color.black).FontSize(24).FontBold().OtherStyle().Parent(scrollContentRight).Build();

        foreach (var group in GroupActions())
        {
            string groupName = group.Key;
            RefreshGroupExpansion(groupName);
            List<ActionItem> actionsInGroup = group.ToList();

            // Create or remove group button depending on whether the group is expanded.
            bool isGroupExpanded = !string.IsNullOrEmpty(searchField.val) || (groupExpanded.ContainsKey(groupName) && groupExpanded[groupName]);
            if (actionsInGroup.Any(a => !script.GetChoosenItem().LinkedActions.Contains(a.Name) && a.Name != GetChosenAction()))
            {
                if (GroupOrAnyActionMatchesSearch(group))
                {
                    CreateGroupButtonAndActionButtons(groupName, false, scrollContentRight);
                }
            }
        }
    }

    private void DrawLinkedActions()
    {
        new TitleBuilder(this, "╔═════════  Linked Actions  ═════════╗").LabelColor(Color.black).FontSize(24).FontBold().OtherStyle().Parent(scrollContentLeft).Build();
        foreach (var group in GroupActions())
        {
            string groupName = group.Key;
            RefreshGroupExpansion(groupName);
            List<ActionItem> actionsInGroup = group.ToList();

            // Create or remove group button depending on whether the group is expanded.
            bool isGroupExpanded = groupExpanded.ContainsKey(groupName) && groupExpanded[groupName];
            if (actionsInGroup.Any(a => script.GetChoosenItem().LinkedActions.Contains(a.Name) && a.Name != GetChosenAction()))
            {
                if (GroupOrAnyActionMatchesSearch(group))
                {
                    CreateGroupButtonAndActionButtons(groupName, true, scrollContentLeft);
                }
            }
        }
    }

    private bool GroupOrAnyActionMatchesSearch(IGrouping<string, ActionItem> group)
    {
        return GroupMatchesSearch(group.Key) || group.ToList().Any(actionItem => actionItem.Name.ToLower().Contains(searchField.val.ToLower()));
    }

    private bool GroupMatchesSearch(string groupName)
    {
        return groupName.ToLower().Contains(searchField.val.ToLower());
    }

    private void CreateGroupButtonAndActionButtons(string groupName, bool isLinked, RectTransform parent)
    {
        string currentSelectedAction = GetChosenAction();
        UnityAction mainAction;
        if (isLinked)
        {
            mainAction = () => RemoveAllActionsFromGroup(groupName);
        }
        else
        {
            mainAction = () => AddAllActionsFromGroup(groupName, currentSelectedAction);
        }

        UnityAction toggleAction = () => ToggleGroup(groupName);

        bool isGroupExpanded = !string.IsNullOrEmpty(searchField.val) || (groupExpanded.ContainsKey(groupName) && groupExpanded[groupName]);

        new TwinButtonBuilder(script, uiElements,
            groupName != "" ? "Group: " + groupName : "Ungrouped",
            mainAction,
            isGroupExpanded ? "-" : "+",
            toggleAction).FirstWidth(0.9f).FirstStart(0).LabelBold(true).Parent(parent).Build();

        if (isGroupExpanded)
        {
            foreach (var actionItem in actionsDictionary.Values.Where(a => a.Group == groupName))
                if ((actionItem.Name != currentSelectedAction) &&
                (isLinked && script.GetChoosenItem().LinkedActions.Contains(actionItem.Name) || (!isLinked && !script.GetChoosenItem().LinkedActions.Contains(actionItem.Name))))
                {
                    if (GroupMatchesSearch(groupName) || actionItem.Name.ToLower().Contains(searchField.val.ToLower()))
                    {
                        CreateActionButton(actionItem, parent);
                    }
                }
        }
    }

    private void AddAllActionsFromGroup(string groupName, string currentSelectedAction)
    {
        if (!string.IsNullOrEmpty(currentSelectedAction))
        {
            ActionItem currentActionItem = script.GetChoosenItem();
            var actionsToAdd = actionsDictionary.Values
                .Where(a => a.Group == groupName && a.Name != currentSelectedAction)
                .Select(a => a.Name);

            foreach (var action in actionsToAdd)
            {
                if (!currentActionItem.LinkedActions.Contains(action))
                {
                    currentActionItem.LinkedActions.Add(action);
                }
            }

            RefreshUI();
        }
    }

    private void RemoveAllActionsFromGroup(string groupName)
    {
        string currentSelectedAction = GetChosenAction();
        if (!string.IsNullOrEmpty(currentSelectedAction))
        {
            var actionsToRemove = actionsDictionary.Values
                .Where(a => a.Group == groupName && a.Name != currentSelectedAction)
                .Select(a => a.Name);

            foreach (var action in actionsToRemove)
            {
                script.GetChoosenItem().LinkedActions.Remove(action);
            }

            RefreshUI();
        }
    }

    private void ToggleGroup(string groupName)
    {
        groupExpanded[groupName] = !groupExpanded[groupName];

        RefreshUI();
    }

    private void CreateActionButton(ActionItem actionItem, RectTransform parent)
    {
        new TwinButtonBuilder(script, uiElements,
        GetButtonName(actionItem),
        () => OnActionButtonClicked(actionItem.Name),
        "←",
        () => { SetChoosenAction(actionItem.Name); })
        .FirstWidth(0.9f).FirstStart(0.05f).Parent(parent)
        .ButtonColor(Colors.DefermineActionButtonColor(actionItem))
        .Build();
    }

    private string GetButtonName(ActionItem actionItem)
    {
        return actionItem.Name + (string.IsNullOrEmpty(actionItem.Layer) ? "" : " (" + actionItem.Layer + ")");
    }

    private void OnActionButtonClicked(string name)
    {
        float timeSinceLastClick = Time.time - lastClickTime;
        if (timeSinceLastClick <= doubleClickThreshold)
        {
            OnActionButtonProcessDoubleClick(name);
            pendingSingleClickName = null;
        }
        else
        {
            pendingSingleClickName = name;
            script.StartCoroutine(WaitAndProcessSingleClick(name));
        }
        lastClickTime = Time.time;
    }

    private IEnumerator WaitAndProcessSingleClick(string name)
    {
        yield return new WaitForSeconds(doubleClickThreshold);

        if (pendingSingleClickName == name)
        {
            OnActionButtonProcessSingleClick(name);
        }
    }

    private void OnActionButtonProcessSingleClick(string name)
    {
        if (string.IsNullOrEmpty(GetChosenAction()))
        {
            SuperController.LogError("No action is currently selected.");
            return;
        }

        ActionItem currentActionItem = script.GetChoosenItem();
        bool isLinked = currentActionItem.LinkedActions.Contains(name);
        if (isLinked)
        {
            currentActionItem.LinkedActions.Remove(name);
        }
        else
        {
            LinkActionItemToName(currentActionItem, name);
        }

        RefreshUI();
        UI.linkedActionsUI.RefreshUI();
    }

    public void LinkActionItemToName(ActionItem actionItem, string nameToAdd)
    {
        if (!ActionItemHasSameActionName(actionItem, nameToAdd))
        {
            if (!string.IsNullOrEmpty(actionItem.Layer))
            {
                actionItem.Layer = "";
            }
            actionItem.LinkedActions.Add(nameToAdd);
        }
        else
        {
            SuperController.LogError("Action with function name = " + actionsDictionary[nameToAdd].Action + " already linked. Cannot have duplicate function names in the same route");
        }
    }

    private void OnActionButtonProcessDoubleClick(string name)
    {
        try
        {
            script.ExecuteAction(actionsDictionary[name]);
        }
        catch (Exception) { }
    }

    private bool ActionItemHasSameActionName(ActionItem actionItem, string nameToAdd)
    {
        foreach (string name in actionItem.LinkedActions)
        {
            if (actionsDictionary[name].Action == actionsDictionary[nameToAdd].Action)
            {
                return true;
            }
        }
        return false;
    }

    private void RefreshGroupExpansion(string groupName)
    {
        if (!groupExpanded.ContainsKey(groupName))
        {
            groupExpanded.Add(groupName, true);
        }
    }

    // GROUP HANDLING

    private void OnSearchChanged(string newSearch)
    {
        RefreshUI();
    }

    private void ExpandAllGroups()
    {
        groupExpanded = groupExpanded.ToDictionary(pair => pair.Key, pair => true);
        RefreshUI();
    }

    private void CloseAllGroups()
    {
        groupExpanded = groupExpanded.ToDictionary(pair => pair.Key, pair => false);
        RefreshUI();
    }
}
